# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from django.test.utils import override_settings
from django.core.mail import send_mail, EmailMessage, EmailMultiAlternatives
import django

from .models import Email


@override_settings(EMAIL_BACKEND='db_email_backend.backend.DBEmailBackend')
class DBEmailBackendTest(TestCase):
    def test_send_mail(self):
        self.assertEqual(Email.objects.count(), 0)

        send_mail('test mail', 'this is just a test', 'jim@bob.com',
                  ['frank@example.com', 'joe@test.com'])

        email = Email.objects.get()
        self.assertEqual(email.subject, 'test mail')
        self.assertEqual(email.body, 'this is just a test')
        self.assertEqual(email.content_subtype, 'plain')
        self.assertEqual(email.from_email, 'jim@bob.com')
        self.assertEqual(email.to, 'frank@example.com; joe@test.com')
        self.assertEqual(email.cc, '')
        self.assertEqual(email.bcc, '')
        self.assertEqual(email.headers, '')

        self.assertEqual(email.attachments.count(), 0)
        self.assertEqual(email.alternatives.count(), 0)

    def test_send_attachments(self):
        self.assertEqual(Email.objects.count(), 0)

        e = EmailMessage('test mail', 'this is just a test', 'jim@bob.com',
                         ['frank@example.com', 'joe@test.com'])
        e.attach_file('README.md')
        e.send()

        email = Email.objects.get()
        self.assertEqual(email.subject, 'test mail')
        self.assertEqual(email.body, 'this is just a test')
        self.assertEqual(email.content_subtype, 'plain')
        self.assertEqual(email.from_email, 'jim@bob.com')
        self.assertEqual(email.to, 'frank@example.com; joe@test.com')
        self.assertEqual(email.cc, '')
        self.assertEqual(email.bcc, '')
        self.assertEqual(email.headers, '')

        attachment = email.attachments.get()
        self.assertEqual(attachment.filename, 'README.md')
        with open('README.md') as f:
            self.assertEqual(attachment.file.read().decode('utf8'), f.read())
        if django.VERSION >= (1, 9):
            self.assertIn(attachment.mimetype, ('application/octet-stream', 'text/markdown'))
        else:
            self.assertEqual(attachment.mimetype, '')

        self.assertEqual(email.alternatives.count(), 0)

    def test_send_alternatives(self):
        self.assertEqual(Email.objects.count(), 0)

        e = EmailMultiAlternatives('test mail', 'this is just a test', 'jim@bob.com',
                                   ['frank@example.com', 'joe@test.com'])
        e.attach_alternative('<h1>Testing stuff</h1>', 'text/html')
        e.send()

        email = Email.objects.get()
        self.assertEqual(email.subject, 'test mail')
        self.assertEqual(email.body, 'this is just a test')
        self.assertEqual(email.content_subtype, 'plain')
        self.assertEqual(email.from_email, 'jim@bob.com')
        self.assertEqual(email.to, 'frank@example.com; joe@test.com')
        self.assertEqual(email.cc, '')
        self.assertEqual(email.bcc, '')
        self.assertEqual(email.headers, '')

        alternative = email.alternatives.get()
        self.assertEqual(alternative.mimetype, 'text/html')
        self.assertEqual(alternative.content, '<h1>Testing stuff</h1>')

        self.assertEqual(email.attachments.count(), 0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import gettext_lazy as _


class Email(models.Model):
    create_date = models.DateTimeField(auto_now_add=True)
    subject = models.TextField(blank=True, verbose_name=_("Subject"))
    body = models.TextField(blank=True, verbose_name=_('Body'))
    content_subtype = models.CharField(max_length=254)
    from_email = models.CharField(max_length=254, blank=True, verbose_name=_('From'))
    to = models.TextField(blank=True, verbose_name=_('To'))
    cc = models.TextField(blank=True, verbose_name=_('CC'))
    bcc = models.TextField(blank=True, verbose_name=_('BCC'))
    headers = models.TextField(blank=True, verbose_name=_('Headers'))
    succeeded = models.BooleanField(default=True, verbose_name=_('Succeeded?'))
    error = models.TextField(blank=True, verbose_name=_('Errors'))

    def __str__(self):
        return '{} - {}, {}'.format(self.subject, self.to, self.create_date)


class EmailAlternative(models.Model):
    email = models.ForeignKey(Email, on_delete=models.CASCADE, related_name='alternatives')
    content = models.TextField(blank=True)
    mimetype = models.CharField(max_length=254, blank=True)

    def __str__(self):
        return '{}: alternative {}'.format(self.email, self.mimetype)


class EmailAttachment(models.Model):
    email = models.ForeignKey(Email, on_delete=models.CASCADE, related_name='attachments')
    filename = models.CharField(max_length=1000, blank=True)
    mimetype = models.CharField(max_length=254, blank=True)
    file = models.FileField(upload_to='db_email_backend')

    def __str__(self):
        return '{}: attachment {}'.format(self.email, self.filename)



from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DBEmailBackendConfig(AppConfig):
    name = 'db_email_backend'
    verbose_name = _('Email Backend')



from django.conf import settings
from django.utils.module_loading import import_string

SMTP_EMAIL_FILTER_FUNCTION_PATH = getattr(settings, 'SMTP_EMAIL_FILTER_FUNCTION_PATH',
                                          'db_email_backend.utils.smtp_filter_email_function')

try:
    email_filter = import_string(SMTP_EMAIL_FILTER_FUNCTION_PATH)
except Exception as e:
    raise e



# Generated by Django 2.2.24 on 2022-04-21 09:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('db_email_backend', '0002_auto_20220420_1246'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='email',
            name='has_errors',
        ),
        migrations.AddField(
            model_name='email',
            name='succeeded',
            field=models.BooleanField(default=True, verbose_name='Succeeded?'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.core.files.base import ContentFile
from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.backends.smtp import EmailBackend as SMTPEmailBackend

from .models import Email, EmailAlternative, EmailAttachment
from .app_settings import email_filter

logger = logging.getLogger('db_mail_backend')


def record_email_message(msg, fail_silently):
    try:
        email = Email.objects.create(
            subject=msg.subject,
            body=msg.body,
            content_subtype=msg.content_subtype,
            from_email=msg.from_email,
            to='; '.join(msg.to),
            cc='; '.join(msg.cc),
            bcc='; '.join(msg.bcc),
            headers='\n'.join('{}: {}'.format(k, v)
                              for k, v in msg.extra_headers.items()),
        )
        # output.append(email)

        alternatives = getattr(msg, 'alternatives', [])
        for content, mimetype in alternatives:
            EmailAlternative.objects.create(
                email=email,
                content=content,
                mimetype=mimetype or '',
            )

        for filename, content, mimetype in msg.attachments:
            attachment = EmailAttachment.objects.create(
                email=email,
                filename=filename,
                mimetype=mimetype or '',
            )
            attachment.file.save(filename, ContentFile(content))
    except:
        email = None
        if not fail_silently:
            raise

    return email


class DBEmailBackend(BaseEmailBackend):
    def send_messages(self, email_messages):
        for msg in email_messages:
            return record_email_message(msg, fail_silently=self.fail_silently)
        return len(email_messages)


class SMTPDBEmailBackend(SMTPEmailBackend):
    """
    This backend is mixture between SMTP and DB mail backend
    It writes to the database then send the email over smtp,
    if any errors happen while sending it is reflected in the email model
    """

    def send_messages(self, email_messages):
        """
        Send one or more EmailMessage objects and return the number of email
        messages sent.
        """
        if not email_messages:
            return 0
        with self._lock:
            new_conn_created = self.open()
            if not self.connection or new_conn_created is None:
                # We failed silently on open().
                # Trying to send would be pointless.
                return 0
            num_sent = 0
            for message in email_messages:
                try:
                    email_inst = record_email_message(message, fail_silently=self.fail_silently)
                except Exception as e:
                    logger.error(e)
                try:
                    if not email_filter(message):
                        continue
                    sent = self._send(message)
                    if sent:
                        num_sent += 1
                except Exception as e:
                    if email_inst:
                        Email.objects.filter(pk=email_inst.pk).update(error=str(e), succeeded=False)
                    raise e
            if new_conn_created:
                self.close()
        return num_sent



#!/usr/bin/env python

import os
import sys

from distutils.core import setup

import db_email_backend

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    sys.exit()

# long_description = open('README.md').read_text()
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.rst").read_text()

setup(
    name='kn-django-db-email-backend',
    version=db_email_backend.__version__,
    description='Django email backend for storing messages to a database.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Ramez Ashraf',
    author_email='ramez@kuwaitnet.com',
    url='https://github.com/KUWAITNET/django-db-email-backend',
    license="MIT License",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
    ],
    packages=[
        'db_email_backend',
        'db_email_backend.migrations',
    ],
    install_requires=[
        "django>=2.2",
        "pytz",
    ],
)



# Generated by Django 2.2.24 on 2022-04-20 09:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('db_email_backend', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='error',
            field=models.TextField(blank=True, verbose_name='Errors'),
        ),
        migrations.AddField(
            model_name='email',
            name='has_errors',
            field=models.BooleanField(default=False, verbose_name='Has Errors'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-03-03 18:09
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('create_date', models.DateTimeField(auto_now_add=True)),
                ('subject', models.TextField(blank=True)),
                ('body', models.TextField(blank=True)),
                ('content_subtype', models.CharField(max_length=254)),
                ('from_email', models.CharField(blank=True, max_length=254)),
                ('to', models.TextField(blank=True)),
                ('cc', models.TextField(blank=True)),
                ('bcc', models.TextField(blank=True)),
                ('headers', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='EmailAlternative',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField(blank=True)),
                ('mimetype', models.CharField(blank=True, max_length=254)),
                ('email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='alternatives', to='db_email_backend.Email')),
            ],
        ),
        migrations.CreateModel(
            name='EmailAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(blank=True, max_length=1000)),
                ('mimetype', models.CharField(blank=True, max_length=254)),
                ('file', models.FileField(upload_to='db_email_backend')),
                ('email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='db_email_backend.Email')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.core.mail import EmailMessage
from django.test import override_settings

from .models import Email, EmailAlternative, EmailAttachment


class EmailAlternativeInline(admin.StackedInline):
    model = EmailAlternative
    readonly_fields = ('mimetype', 'content')
    extra = 0
    max_num = 0
    can_delete = False


class EmailAttachmentInline(admin.StackedInline):
    model = EmailAttachment
    readonly_fields = ('filename', 'mimetype', 'file')
    extra = 0
    max_num = 0
    can_delete = False


class EmailAdmin(admin.ModelAdmin):
    fields = (
        ('from_email', 'create_date', 'content_subtype'),
        ('to', 'cc', 'bcc'),
        'subject', 'body', 'headers', 'succeeded', 'error')
    readonly_fields = (
        'create_date', 'from_email', 'to', 'cc', 'bcc', 'subject', 'body', 'content_subtype', 'headers', 'succeeded',
        'error')
    list_display = (
    'subject', 'to', 'from_email', 'create_date', 'attachment_count', 'alternative_count', 'succeeded', 'error')
    list_filter = ('succeeded', 'content_subtype',)
    date_hierarchy = 'create_date'
    search_fields = ('to', 'from_email', 'cc', 'bcc', 'subject', 'body')
    inlines = (EmailAlternativeInline, EmailAttachmentInline)

    actions = ['send_mail']

    def get_fields(self, request, obj=None):
        fields = super().get_fields(request, obj)
        if not request.user.is_superuser:
            fields = list(fields)
            fields.remove('body')
        return fields

    @override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend')
    def send_mail(self, request, queryset):
        for row in queryset:
            msg = EmailMessage()
            msg.subject = row.subject
            msg.body = row.body
            msg.content_subtype = row.content_subtype
            msg.from_email = row.from_email
            msg.to = [x for x in row.to.split('; ') if x]
            msg.cc = [x for x in row.cc.split('; ') if x]
            msg.bcc = [x for x in row.bcc.split('; ') if x]

            msg.send(fail_silently=False)

        self.message_user(request, f'{queryset.count()} Emails sent', 25)

    send_mail.short_description = "Send Actual Email via SMTP"

    def attachment_count(self, instance):
        return instance.attachments.count()

    attachment_count.short_description = 'Attachments'

    def alternative_count(self, instance):
        return instance.alternatives.count()

    alternative_count.short_description = 'Alternatives'

    def has_add_permission(self, request, obj=None):
        return False


admin.site.register(Email, EmailAdmin)
